# 视频编码和解码
#### 目录介绍
- **1.做Android视频编辑的可行性开源方案**
- 1.1 第三方流行框架
- 1.2 Android自带框架
- 1.3 这些方案功能分析
- 1.4 这些方案学习门槛分析
- **2.对音频进行编码和解码**
- 2.1 MediaCodec基本介绍
- 2.2 MediaCodec核心原理
- **3.如何通俗理解编码和解码**
- 3.1 什么是编码
- 3.2 什么是解码
- **4.对音频如何混音添加背景音乐**
- 4.1 待更新实现



### 1.做Android视频编辑的可行性开源方案
#### 1.1 第三方流行框架
- 大家熟知的ffmpeg,将ffmpeg移植到anroid平台，编译成so文件，由jni 调用，可以实现音视频的分离、裁剪、拼合、加字幕、滤镜等功能。


#### 1.2 Android自带框架
- android 自带的MediaCodec 框架，MediaCodec框架底层调用的是StageFright库，StageFright库是默认封装在android系统里面的
- 官方的 MediaCodec API 虽然支持硬件编解码加速，但是问题和局限还是很多的，一方面是只能在 Android 4.1 以上机型上才能使用，另一方面，由于 Android 手机种类繁多，厂商对底层源码的修改各不相同，导致 MediaCodec API 在实际使用中，会遇到很多坑，有很多兼容性的问题，因此，可以考虑采用第三方的编解码库。


#### 1.3 这些方案功能分析
- **1.3.1 ffmpeg功能分析**
- ffmpeg 无疑排第一位，他集合了视频编解码、视频滤镜、流媒体推流、音频各种特效等等，基本上你能想到的功能都在里面。
- 在网上看到说，国内像暴风，某某影音，某某视频等等都用到了该开源框架，

- **1.3.1 MediaCodec功能分析**
- MediaCodec涵盖了音视频解复用、音频解码、视频解码、音频编码、视频编码、音视频合并的整个流程。跟ffmpeg相比，MediaCodec 更接近底层硬件。这个方案如果想要实现视频的滤镜、字幕、拼接等功能的话，需要自己配合OpenGL ES 来实现，另外，音视频拼接的话，要考虑到不同音频采样率的重采样问题，音频重采用问题，需要懂得傅立叶变换相关的离散信号变换方法，如果要实现音频特效，如变声、均衡器的话，也需要懂得上述信号变换方法。


#### 1.4 这些方案学习门槛分析
- 如果只是做视频转码、加文字、图片特效等，ffmpeg和MediaCodec 旗鼓相当。如果是要拼接视频、做音频的变声、均衡器特效的话，MediaCodec是难度最高的，因为这一切需要你从底层原理做起。
- 运行效率:MediaCodec硬解硬编最快，ffmpeg硬解硬编方案稍慢。
- 稳定性: MediaCodec和ffmpeg 的硬解硬编方案旗鼓相当。
- 打包占用空间:国内最得最好的ffmpeg硬解硬编方案，其so文件在10.几M,MediaCodec由于是纯java 代码，占用空间很容易做到几百K甚至几十K。


### 2.对音频进行编码
#### 2.1 MediaCodec基本介绍
- 提供了一套访问 Android 底层多媒体模块的接口，主要是音视频的编解码接口
- Android 应用层统一由 MediaCodec API 来提供各种音视频编解码功能，由参数配置来决定采用何种编解码算法、是否采用硬件编解码加速等等

#### 2.2 MediaCodec核心原理
- **2.2.1 MediaCodec 使用的基本流程**
```
- createEncoderByType/createDecoderByType
- configure
- start
- while(1) {
    - dequeueInputBuffer
    - queueInputBuffer
    - dequeueOutputBuffer
    - releaseOutputBuffer
}
- stop
- release
```

- **2.2.2 Buffer队列示意图**
- ![image](http://s3.51cto.com/wyfs02/M00/7E/7B/wKioL1cCSojSz4HdAAD0xxQxvwg230.png)

- **2.2.3 缓冲处理队列的逻辑**
- MediaCodec 架构上采用了2个缓冲区队列，异步处理数据，下面描述的 Client 和 MediaCodec 模块是并行工作的（注：这里的 Client 就是指 “开发者，API 的使用者”）：
    * （1）Client 从 input 缓冲区队列申请 empty buffer [dequeueInputBuffer]
    * （2）Client 把需要编解码的数据拷贝到 empty buffer，然后放入 input 缓冲区队列 [queueInputBuffer] 
    * （3）MediaCodec 模块从 input 缓冲区队列取一帧数据进行编解码处理
    * （4）编解码处理结束后，MediaCodec 将原始数据 buffer 置为 empty 后放回 input 缓冲区队列，将编解码后的数据放入到 output 缓冲区队列
    * （5）Client 从 output 缓冲区队列申请编解码后的 buffer [dequeueOutputBuffer]
    * （6）Client 对编解码后的 buffer 进行渲染/播放
    * （7）渲染/播放完成后，Client 再将该 buffer 放回 output 缓冲区队列 [releaseOutputBuffer]
- MediaCodec 在架构上，其实是采用了一种基于“环形缓冲区”的“生产者-消费者”模式，它设计了 2 个基于 idx 序号的“环形缓冲区” ，注意，是 2 个，一个在 input 端， 一个在 output 端。




### 5.其他问题说明
#### 5.1 版本更新情况
- v1.0.0 2017年12月8日
- v1.0.1 2018年2月2日


#### 5.2 参考链接
- Android硬编码——音频编码、视频编码及音视频混合：http://blog.csdn.net/junzia/article/details/54018671
- android MediaCodec 音频编解码的实现——转码:https://www.cnblogs.com/Sharley/p/5964490.html
- Android音频开发（5）：音频数据的编解码：http://blog.51cto.com/ticktick/1760191
- Android 音视频编辑经验总结及开源工程分享：https://www.cnblogs.com/jerrychen33/p/8148993.html

























